import numpy as np
from cgibbs_inf import inf


class lda_cython:
    """
    Input parameters:
        K: number of topics
        alpha: Parameter for Dirichlet distribution for document topic distribution.
        beta: Parameter for Dirichlet distribution for topic word distribution.
        docs: Documents encoded with index.
        vocab_len: Length of vocabulary.
        iteration: Number of Iteration.
    """

    def __init__(self, K=25, alpha=0.5, beta=0.5, docs=None, vocab_len=None, iteration=10):
        """Initialize the parameters"""

        self.K, self.α, self.β, self.docs, self.vocab_len, self.iteration = K, alpha, beta, docs, vocab_len, iteration
        self.docs_len = len(self.docs)

        self.topic_doc_word = np.zeros(docs.shape, dtype=int)
        self.topic_doc_word.fill(-1)

        self.cnt_doc_topic = np.zeros((self.docs_len, self.K)) + alpha
        self.cnt_topic_word = np.zeros((self.K, vocab_len)) + beta
        self.cnt_topic = np.zeros(self.K) + vocab_len * beta

        for d_i, doc in enumerate(docs):
            for idx, do in enumerate(doc):
                if do < 0:
                    continue
                i = int(do)
                z = np.random.randint(0, K)

                # Randomly assigned topic
                self.topic_doc_word[d_i, idx] = z

                # Increase the counters accordingly
                self.cnt_doc_topic[d_i, z] += 1
                self.cnt_topic_word[z, i] += 1
                self.cnt_topic[z] += 1

        print(self.topic_doc_word)

    def inference(self):
        """
        Cython Optimized inference function.
        Pseudocode Available in the report.
        """

        inf(self.K, self.α, self.β, self.docs, self.vocab_len, self.iteration, self.topic_doc_word, self.cnt_doc_topic,
            self.cnt_topic_word, self.cnt_topic)

    def topics_in_document_distribution(self):
        """Output the distribution of topics in documents."""

        topic_distribution = np.zeros(self.cnt_doc_topic.shape)
        for idx, row in enumerate(self.cnt_doc_topic):
            topic_distribution[idx] = self.cnt_doc_topic[idx] / self.cnt_doc_topic[idx].sum()
        return topic_distribution

    def words_in_topic_distributions(self):
        """Output the distribution of words in topics."""

        return self.cnt_topic_word / self.cnt_topic.reshape(self.cnt_topic_word.shape[0], 1)



import unittest

from ktlda.utils import prepare_documents

TOY_DOCS = ['sunny hot heat frost icy icy',
            'sunny sunny hot heat sunny hot sunny hot',
            'sunny sunny hot hot hot',
            'sunny hot heat heat heat sunny hot heat',
            'icy frost icy frost frost icy icy icy frost frost icy icy',
            'frost frost icy frost frost icy',
            'frost icy icy icy icy icy frost icy']


class TestUtils(unittest.TestCase):
    def test_correct(self):
        print(prepare_documents(TOY_DOCS))


if __name__ == '__main__':
    unittest.main()



import numpy as np


def prepare_documents(documents):
    """
    :param documents:
    :return: (doc_lda, V). doc_lda: documents represented by word index.
        V: word to index mapping
    """
    docs = []

    for doc in documents:
        docs.append(doc.split())

    dicts = []
    for d in docs:
        for w in d:
            dicts.append(w)

    V = dicts = list(set(dicts))
    dicts = dict([(w, idx) for idx, w in enumerate(dicts)])

    for i, d in enumerate(docs):
        for j, w in enumerate(d):
            docs[i][j] = dicts[docs[i][j]]

    doc_lda = docs

    b = np.zeros([len(doc_lda), len(max(doc_lda, key=lambda x: len(x)))], dtype=int)
    b.fill(-1)
    for i, j in enumerate(doc_lda):
        b[i][0:len(j)] = j

    doc_lda = b
    return doc_lda, V


def ldagen(vocab, alpha, beta, lam, ndoc=10, rnd_seed=663):
    """
    Performs the following generative process of LDA:
    - sample document length: N ~ Poisson(ξ)
    - sample document-topic distribution: θ ~ Dir(α)
    - for each word w_n:
        - sample topic: k ~ Categorical(θ)
        - sample word: w ~ Categorical(β_k)

    :param vocab: vocabulary. A list of words.
    :param alpha: param for Dir distribution of document-topic. Shape (num_D, num_T)
    :param beta: param for Dir distribution of topic-word. Shape (num_T, len(vocab))
    :param lam: param for Poisson distribution
    :param rnd_seed: seed for random generator
    :return docs: documents
    """
    np.random.seed(rnd_seed)
    docs = []
    for i in range(ndoc):
        curr_doc = []
        num_words = np.random.poisson(lam)
        theta = np.random.dirichlet(alpha)
        for _ in range(num_words):
            k = np.argmax(np.random.multinomial(1, theta))
            w = None
            curr_doc.append(w)
    return docs


if __name__ == '__main__':
    vocab = ['good', 'great', 'bad', 'terrible']
    alpha = []
    ldagen(vocab, )



"""
Adapt to sklearn-like API for client usage
"""

from ktlda.model import lda_cython
import numpy as np
from collections import Counter
from tqdm.autonotebook import tqdm


class KtLDA:
    """
    LDA topic model via Collapsed Gibbs Sampling
    """

    def __init__(self, n_components, alpha=0.5, beta=0.5, iterations=10, max_vocab=5000, random_state=None):
        """

        :param n_components: number of topics
        :param alpha: Dirichlet parameter for document-topic distribution
        :param beta: Dirichlet parameter for topic and word distribution
        :param iterations: number of iterations
        :param max_vocab: number of most frequent words to preserve. Less frequent words
                          will be dropped
        :param random_state: sets the seed of numpy random number generator if not None
        """
        self.n_components = n_components
        self.alpha = alpha
        self.beta = beta
        self.iterations = iterations
        self.max_vocab = max_vocab
        if random_state:
            np.random.seed(random_state)
        self.model = None
        self.doc_topic_dist = None
        self.topic_word_dist = None

    def _convert_docs(self, documents, max_vocab=5000):
        """
        Convert documents to acceptable input to our algorithm
        :return: (document matrix, vocabulary dict)
        """
        print('word counting')
        wordcount = Counter()
        for doc in tqdm(documents):
            wordcount += Counter(doc.split())

        vocab, _ = zip(*wordcount.most_common(max_vocab))
        dicts = dict([(w, idx) for idx, w in enumerate(vocab)])

        print('transform doc while removing least frequent words')
        docs = []
        vocabset = set(vocab)
        for i, d in tqdm(enumerate(documents)):
            docs.append([])
            for j, w in enumerate(d.split()):
                if w in vocabset:
                    docs[i].append(dicts[w])

        doc_mat = np.zeros([len(docs), len(max(docs, key=lambda x: len(x)))], dtype=int)
        doc_mat.fill(-1)
        for i, words in enumerate(docs):
            for j, w in enumerate(words):
                doc_mat[i, j] = w
        return doc_mat, dicts

    def fit(self, documents):
        doc_mat, dicts = self._convert_docs(documents, self.max_vocab)
        self.model = lda_cython(K=self.n_components, alpha=self.alpha, beta=self.beta, docs=doc_mat,
                                vocab_len=len(dicts),
                                iteration=self.iterations)
        self.model.inference()
        self.doc_topic_dist = self.model.topics_in_document_distribution()
        self.topic_word_dist = self.model.words_in_topic_distributions()



import setuptools
from distutils.extension import Extension
# from distutils.core import setup
# from Cython.Build import cythonize

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ktLDA",
    version="0.0.6",
    author="Kehan (kehanLyu) & Tiangang (cPolaris)",
    author_email="",
    description="An implementation of latent Dirichlet allocation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cPolaris/ktLDA",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    ext_modules=[Extension("cgibbs_inf", ["ktlda/cgibbs_inf.c"])]
)

# setup(
#     ext_modules=cythonize("ktlda/cgibbs_inf.pyx")
# )
